<?php
namespace Magenest\Salesforce\Model\ResourceModel;

use Magenest\Salesforce\Helper\Constant;
/**
 * Class Queue
 * @package Magenest\Salesforce\Model\ResourceModel
 */
class Queue extends \Magento\Framework\Model\ResourceModel\Db\AbstractDb
{
    protected function _construct()
    {
        $this->_init('magenest_salesforce_queue', 'id');
    }

    /**
     * @param $collectionArr
     *
     * @throws \Exception
     */
    public function deleteMultiQueues($collectionArr)
    {
        $collectionSize = count($collectionArr);
        if (!is_array($collectionArr) || $collectionSize == 0) {
            return;
        }
        $collectionIds = implode(', ', $collectionArr);
        $this->getConnection()->delete(
            $this->getMainTable(),
            "{$this->getIdFieldName()} in ({$collectionIds})"
        );
    }

    /**
     * @param $queueArr
     * @return bool
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Throwable
     */
    public function insertMultiple($queueArr)
    {
        $connection = $this->getConnection();
        try {
            $connection->beginTransaction();
            $this->getConnection()->insertMultiple(
                $this->getMainTable(),
                $queueArr
            );
            $connection->commit();
            return true;
        } catch (\Throwable $exception) {
            $connection->rollBack();
            throw $exception;
        }
    }

    /**
     * @param $type
     * @return array
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function fetchEntityIdByType($type)
    {
        return $this->getConnection()->fetchPairs(
            $this->getConnection()->select()
                ->from($this->getMainTable(), ['id', 'entity_id'])
                ->where('type = ?', rtrim($type, 's'))
        );
    }

    /**
     * @param $data
     * @param $type
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function publishData($data, $type)
    {
        if (is_array($data)) {
            $chunks = array_chunk($data, Constant::SIZE);
            foreach ($chunks as $chunk) {
                $dataRaw = $this->convertData($chunk, $type);
                $this->insertMultiple($dataRaw);
            }
        }
    }

    /**
     * @param $data
     * @param $type
     * @return array
     */
    protected function convertData($data, $type)
    {
        $results = [];
        foreach ($data as $id) {
            $results[] = [
                'type' => $type,
                'entity_id' => $id
            ];
        }
        return $results;
    }

    /**
     * @param array $entityIds
     * @return array
     * @throws \Exception
     */
    public function selectIdByEntityId(array $entityIds)
    {
        try {
            $connection = $this->getConnection();
            $sql = $connection->select()
                ->from($this->getMainTable(), ['id'])
                ->where('entity_id IN (?)', $entityIds)
                ->distinct();
            return $connection->fetchCol($sql);
        } catch (\Throwable $exception) {
            throw new \Exception($exception->getMessage());
        }
    }

    public function updateTypeByIds(array $ids, $type)
    {
        $connection = $this->getConnection();
        try {
            $condition = ['id IN (?)' => $ids];
            $connection->beginTransaction();
            $connection->update($this->getMainTable(), ['type' => $type], $condition);
            $connection->commit();
        } catch (\Throwable $exception) {
            $connection->rollBack();
            throw new \Exception($exception->getMessage());
        }
    }
}
